#
# Copyright (C) 2010 Red Hat, Inc.
#
# Authors:
# Thomas Woerner <twoerner@redhat.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

ALREADY_ENABLED   =   -1
NOT_ENABLED       =   -2
COMMAND_FAILED    =   -3
NO_IPV6_NAT       =   -4
PANIC_MODE        =   -5
ZONE_ALREADY_SET  =   -6
UNKNOWN_INTERFACE =   -7
ZONE_CONFLICT     =   -8
IMMUTABLE         =   -9
BUILTIN_CHAIN     =  -10
EBTABLES_NO_REJECT = -11
NOT_OVERLOADABLE  =  -12

INVALID_ACTION    =  -20
INVALID_SERVICE   =  -21
INVALID_PORT      =  -22
INVALID_PROTOCOL  =  -23
INVALID_INTERFACE =  -24
INVALID_ADDR      =  -25
INVALID_FORWARD   =  -26
INVALID_ICMPTYPE  =  -27
INVALID_TABLE     =  -28
INVALID_CHAIN     =  -29
INVALID_TARGET    =  -30
INVALID_IPV       =  -31
INVALID_ZONE      =  -32
INVALID_PROPERTY  =  -33
INVALID_VALUE     =  -34
INVALID_OBJECT    =  -35

MISSING_TABLE     =  -40
MISSING_CHAIN     =  -41
MISSING_PORT      =  -42
MISSING_PROTOCOL  =  -43
MISSING_ADDR      =  -44

NOT_RUNNING       =  -98
NOT_AUTHORIZED    =  -99
UNKNOWN_ERROR     = -100

import sys

class FirewallError(Exception):
    mod = sys.modules[__module__]
    errors = dict([(getattr(mod,varname),varname)
                   for varname in dir(mod)
                   if not varname.startswith("_")])
    codes = dict([(errors[code],code) for code in errors])

    def __init__(self, code, msg=None):
        self.code = code
        self.msg = msg

    def __str__(self):
        if self.msg:
            return "%s: %s" % (self.errors[self.code], self.msg)
        return self.errors[self.code]

    def get_code(msg):
        if ":" in msg:
            idx = msg.index(":")
            ecode = msg[:idx]
        else:
            ecode = msg
        return FirewallError.codes[ecode]
    get_code = staticmethod(get_code)
